#####################	SETS
param NM>=1, integer ;
param NG>=1, integer ;
param nk1>=1, integer ;
param nk2>=1, integer ;
param nk3>=1, integer ;
param nk6>=1, integer ;
param nk7>=1, integer ;
param NK1:= 2^nk1, integer ;	## numero de tramos
param NK2:= 2^nk2, integer ;
param NK3:= 2^nk3, integer ;
param NK6:= 2^nk6, integer ;
param NK7:= 2^nk7, integer ;

set PRODUCTS ;
## Etapas
set STAGES1 ;		## etapas batch
set STAGES2 ;		## etapas semicontinuas
set STAGES3 ;		## etapas cromatograficas
set STAGES4 ;		## ultima etapa cromatografica
set STAGES5:= STAGES3 union STAGES4 ;
set STAGES:= STAGES1 union STAGES2 union STAGES3 union STAGES4 ;
## unidades
set UNIT1:= STAGES1 union STAGES2 union STAGES3 union STAGES4 ;
set UNIT2 within {STAGES2 union STAGES3 union STAGES4} ;			# etapas con tanque de permeado
set UNIT3:= STAGES2 union STAGES3 union STAGES4 ;
## Etapas de acuerdo con cada producto
set PSTAGES1 within {PRODUCTS, STAGES1} ;
set PSTAGES2 within {PRODUCTS, STAGES2} ;
set PSTAGES3 within {PRODUCTS, STAGES3} ;
set PSTAGES4 within {PRODUCTS, STAGES4} ;
set PSTAGES5:= PSTAGES3 union PSTAGES4 ;
## unidades de acuerdo con cada producto
set PUNIT2S within {PRODUCTS, STAGES2} ;					# unidades 2 en etapas semicontinuas
set PUNIT1:= PSTAGES1 union PSTAGES2 union PSTAGES3 union PSTAGES4 ;
set PUNIT2:= PUNIT2S union PSTAGES3 union PSTAGES4 ;
set PUNIT3:= PSTAGES2 union PSTAGES3 union PSTAGES4 ;
## unidades con T0 y con T1
set PSTAGEST0:= PSTAGES1 union PSTAGES3 union PSTAGES4 ;
set PSTAGEST1:= PSTAGES2 union PSTAGES3 ;
## unidades en paralelo
set UNITSM:= 1..NM ordered ;
set UNITSG:= 1..NG ordered ;
## transformaciones MIP
set TMIP1:= 1..NK1 ;	
set TMIP2:= 1..NK2 ;
set TMIP3:= 1..NK3 ;
set TMIP6:= 1..NK6 ;
set TMIP7:= 1..NK7 ;
set TMIP1U:= 1..NK1+1 ;		## conjunto de puntos
set TMIP2U:= 1..NK2+1 ;
set TMIP3U:= 1..NK3+1 ;
set TMIP6U:= 1..NK6+1 ;
set TMIP7U:= 1..NK7+1 ;

##### PARAMETERS
## Coeficientes de costo
param c1{j in UNIT1} ;
param c2{j in UNIT2} ;
param c3{j in UNIT3} ;
param cmax := max(max{k in UNIT1} c1[k],max{k in UNIT2} c2[k],max{k in UNIT3} c3[k]) ;
param C1{j in UNIT1} := c1[j]/(cmax) ;
param C2{j in UNIT2} := c2[j]/(cmax) ;
param C3{j in UNIT3} := c3[j]/(cmax) ;
param gamma1{j in UNIT1} ;
param gamma2{j in UNIT2} ;
param gamma3{j in UNIT3} ;

## Factores de tamaño y tiempo
param S1{PUNIT1} ;
param S2{PUNIT2} ;
param S3{PSTAGES5} ;
param s1{(i,j) in PUNIT1} := log(S1[i,j]) ;
param s2{(i,j) in PUNIT2} := log(S2[i,j]) ;
param s3{(i,j) in PSTAGES5} := log(S3[i,j]) ;
param T0{PSTAGEST0} ;
param T1{PSTAGEST1} ;
param t0{(i,j) in PSTAGEST0} := log(T0[i,j]) ;
param t1{(i,j) in PSTAGEST1} := log(T1[i,j]) ;

## coeficientes de horizonte de produccion
param d{i in PRODUCTS} ;
param delta ;
param coef_delta ;
param rho ;
param NR ;
param RHO := rho/cmax ;

## limites de variables
param y1LO{j in UNIT1} ;
param y1UP{j in UNIT1} ;
param y2LO{j in UNIT2} ;
param y2UP{j in UNIT2} ;
param y3LO{j in UNIT3} ;
param y3UP{j in UNIT3} ;
param y4LO{i in PRODUCTS} ;
param y4UP{i in PRODUCTS} ;
param y5LO{i in PRODUCTS} ;
param y5UP{i in PRODUCTS} ;

param y1min{j in UNIT1} ;
param y2min{j in UNIT2} ;
param y3min{j in UNIT3} ;
param y4min{i in PRODUCTS} ;
param y5min ;

param y1lo{j in UNIT1} ;
param y1up{j in UNIT1} ;
param y2lo{j in UNIT2} ;
param y2up{j in UNIT2} ;
param y3lo{j in UNIT3} ;
param y3up{j in UNIT3} ;

param y4lo{i in PRODUCTS} ;
param y4up{i in PRODUCTS} ;

param y5lo{i in PRODUCTS} ;
param y5up{i in PRODUCTS} ;

param z1lo{j in UNIT1} ;
param z1up{j in UNIT1} ;
param z2lo{j in UNIT2} ;
param z2up{j in UNIT2} ;
param z3lo{j in UNIT3} ;
param z3up{j in UNIT3} ;

param z6lo{(i,j) in PSTAGES3} ;
param z6up{(i,j) in PSTAGES3} ;

param z7lo{i in PRODUCTS} ;
param z7up{i in PRODUCTS} ;

param v1lo{j in UNIT1} ;
param v1up{j in UNIT1} ;
param v2lo{j in UNIT2} ;
param v2up{j in UNIT2} ;
param v3lo{j in UNIT3} ;
param v3up{j in UNIT3} ;
param v7lo{i in PRODUCTS} ;
param v7up{i in PRODUCTS} ;

## constantes para transformaciones MIP
param a1{j in UNIT1}	:=	(z1up[j] - z1lo[j])/NK1 ;
param b1{j in UNIT1, k in TMIP1U}	:=	z1lo[j] + (k-1)*a1[j] ;
param beta1{j in UNIT1, k in TMIP1U}	:=	C1[j]*exp(b1[j,k]) ;
param alpha1{j in UNIT1, k in TMIP1}	:=	(beta1[j,k+1]-beta1[j,k])/a1[j] ;

param a2{j in UNIT2}	:=	(z2up[j] - z2lo[j])/NK2 ;
param b2{j in UNIT2, k in TMIP2U}	:=	z2lo[j] + (k-1)*a2[j] ;
param beta2{j in UNIT2, k in TMIP2U}	:=	C2[j]*exp(b2[j,k]) ;
param alpha2{j in UNIT2, k in TMIP2}	:=	(beta2[j,k+1]-beta2[j,k])/a2[j] ;

param a3{j in UNIT3}	:=	(z3up[j] - z3lo[j])/NK3 ;
param b3{j in UNIT3, k in TMIP3U}	:=	z3lo[j] + (k-1)*a3[j] ;
param beta3{j in UNIT3, k in TMIP3U}	:=	C3[j]*exp(b3[j,k]) ;
param alpha3{j in UNIT3, k in TMIP3}	:=	(beta3[j,k+1]-beta3[j,k])/a3[j] ;

param a6{(i,j) in PSTAGES3}	:=	(z6up[i,j]-z6lo[i,j])/NK6 ; 
param b6{(i,j) in PSTAGES3, k in TMIP6U}	:=	z6lo[i,j] + (k-1)*a6[i,j] ;
param beta6{(i,j) in PSTAGES3, k in TMIP6U}	:=	log(T0[i,j]+exp(t1[i,j]+b6[i,j,k])) ;
param alpha6{(i,j) in PSTAGES3, k in TMIP6}	:=	(beta6[i,j,k+1]-beta6[i,j,k])/a6[i,j] ;

param a7{i in PRODUCTS}		:=	(z7up[i] - z7lo[i])/NK7 ;
param b7{i in PRODUCTS, k in TMIP7U}		:=	z7lo[i] + (k-1)*a7[i] ;
param beta7{i in PRODUCTS, k in TMIP7U}		:=	exp(b7[i,k]+log(d[i])-log(coef_delta)) ;
param alpha7{i in PRODUCTS, k in TMIP7}		:=	(beta7[i,k+1] - beta7[i,k])/a7[i] ;

##### VARIABLES
var r>= 0 , <= NR ;
var y1{j in UNIT1}	>= y1lo[j] , <= y1up[j] ;
var y2{j in UNIT2}	>= y2lo[j] , <= y2up[j] ;
var y3{j in UNIT3}	>= y3lo[j] , <= y3up[j] ;
var y4{i in PRODUCTS}	>= y4lo[i] , <= y4up[i] ;
var y5{i in PRODUCTS}	>= y5lo[i] , <= y5up[i] ;
var y6{j in STAGES, k in UNITSG} binary , >= 0 , <= 1 ;
var y7{j in STAGES, k in UNITSM} binary , >= 0 , <= 1 ;
var x1{j in STAGES} >= log(1) , <= log(NG) ;
var x2{j in STAGES} >= log(1) , <= log(NM) ;
var v1{j in UNIT1} <= v1up[j] ;
var v2{j in UNIT2} <= v2up[j] ;
var v3{j in UNIT3} <= v3up[j] ;
var v7{i in PRODUCTS} <= v7up[i] ;

##### OBJECTIVE
minimize cost:

	sum{j in STAGES1} v1[j]
	+ sum{j in STAGES2} ( v1[j] + v3[j] )
	+ sum{j in STAGES2 inter UNIT2} (v2[j])
	+ sum{j in STAGES5} (v1[j] + v2[j] + v3[j])
	+ r*RHO*coef_delta ;

##### CONSTRAINTS
## Restricciones para etapas batch
subject to C1_BATCH{(i,j) in PSTAGES1}:
	y1[j] + x1[j] >= s1[i,j] + y4[i] ;
subject to C2_BATCH{(i,j) in PSTAGES1}:
	y5[i] + x2[j] >= t0[i,j] ;

## Restricciones para unidades semicontinuas
subject to C1_SEMIC{(i,j) in PSTAGES2}:
	y1[j] >= s1[i,j] + y4[i] ;
subject to C2_SEMIC{(i,j) in PUNIT2S}:
	y2[j] >= s2[i,j] + y4[i] ;
subject to C3_SEMIC{(i,j) in PSTAGES2}:
	y5[i] + y3[j] + x1[j] + x2[j] >= t1[i,j] + y4[i] ;

### Restricciones para etapas cromatograficas
subject to C1_CHROM{(i,j) in PSTAGES5}:
	y1[j] >= s1[i,j] + y4[i] ;
subject to C2_CHROM{(i,j) in PSTAGES5}:
	y2[j] >= s2[i,j] + y4[i] ;
subject to C3_CHROM{(i,j) in PSTAGES5}:
	y3[j] + x1[j] >= s3[i,j] + y4[i] ;
subject to C4_CHROM{(i,j) in PSTAGES3, k in TMIP6}:
	y5[i] + x2[j] >= alpha6[i,j,k]*(y4[i]-x1[j]-y3[j]) - alpha6[i,j,k]*b6[i,j,k] + beta6[i,j,k] ;
subject to C5_CHROM{(i,j) in PSTAGES4}:
	y5[i] + x2[j] >= t0[i,j];

## Definicion de variables v
subject to C1v1{j in STAGES1, k in TMIP1}:
	v1[j] >= alpha1[j,k]*(x1[j]+x2[j]+gamma1[j]*y1[j]) - alpha1[j,k]*b1[j,k] + beta1[j,k] ;
subject to C2v1{j in UNIT1 diff STAGES1, k in TMIP1}:
	v1[j] >= alpha1[j,k]*(x2[j]+gamma1[j]*y1[j]) - alpha1[j,k]*b1[j,k] + beta1[j,k] ;
subject to C1v2{j in UNIT2, k in TMIP2}:
	v2[j] >= alpha2[j,k]*(x2[j]+gamma2[j]*y2[j]) - alpha2[j,k]*b2[j,k] + beta2[j,k] ;
subject to C1v3{j in UNIT3, k in TMIP3}:
	v3[j] >= alpha3[j,k]*(x1[j]+x2[j]+gamma3[j]*y3[j]) - alpha3[j,k]*b3[j,k] + beta3[j,k] ;
subject to C1v7{i in PRODUCTS,k in TMIP7}:
	v7[i] >= alpha7[i,k]*(y5[i]-y4[i]) - alpha7[i,k]*b7[i,k] + beta7[i,k] ;

## Horizonte de planificacion
subject to CHP:
	sum{i in PRODUCTS} v7[i] <= (delta/coef_delta) + r ;

## Variables binarias para numero de etapas en paralelo
subject to EP1{j in STAGES}:
	x1[j] = sum{k in UNITSG} y6[j,k]*log(k) ;
subject to EP2{j in STAGES}:
	sum{k in UNITSG} y6[j,k] = 1 ;
subject to EP3{j in STAGES}:
	x2[j] = sum{k in UNITSM} y7[j,k]*log(k) ;
subject to EP4{j in STAGES}:
	sum{k in UNITSM} y7[j,k] = 1 ;
